#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "parsermaker.h"

FILE *ofile;

void GenSwitch(short input, short level);
void GenPTable(short input, short level);

void main(void)
{
	short	i;
	short	t = 0;

	// Output Enums
	ofile = fopen("U-SFUNCS.H","w");

	t = 0;
	fprintf(ofile, "// Keyword Function Enums\n");
	fprintf(ofile, "// Automatically generated by ParserMaker.c\n");
	fprintf(ofile, "//\n");
	fprintf(ofile, "enum {\n");
	for (i = 0; keyWords[i].keyWord; ++i) {
		if (i == 0 || strcmp(keyWords[i].funcName,keyWords[i-1].funcName)) {
			if ((t++) == 0)
				fprintf(ofile, "\tsf_%s, ",&keyWords[i].funcName[2]);	
			else if (t < 5)
				fprintf(ofile, "sf_%s, ",&keyWords[i].funcName[2]);	
			else {
				fprintf(ofile, "sf_%s,\n",&keyWords[i].funcName[2]);	
				t = 0;
			}
		}
	}
	fprintf(ofile, "sf_NbrScriptFunctions };\n\n");


	// Output Function Templates
	fprintf(ofile, "// Function Templates\n");
	fprintf(ofile, "// Automatically generated by ParserMaker.c\n");
	fprintf(ofile, "//\n");
	for (i = 0; keyWords[i].keyWord; ++i) {
		if (i == 0 || strcmp(keyWords[i].funcName,keyWords[i-1].funcName)) {
			fprintf(ofile, "void %s();\n",keyWords[i].funcName);	
		}
	}
	fprintf(ofile, "\n\n");

	// Output Function Table
	t = 0;
	fprintf(ofile, "#if _U_SFUNC\n");
	fprintf(ofile, "// Keyword Function Table\n");
	fprintf(ofile, "// Automatically generated by ParserMaker.c\n");
	fprintf(ofile, "//\n");
	fprintf(ofile, "void (*SFuncArray[sf_NbrScriptFunctions])() = {\n");
	for (i = 0; keyWords[i].keyWord; ++i) {
		if (i == 0 || strcmp(keyWords[i].funcName,keyWords[i-1].funcName)) {
			if ((t++) == 0)
				fprintf(ofile, "\t%s, ",keyWords[i].funcName);	
			else if (t < 5)
				fprintf(ofile, "%s, ",keyWords[i].funcName);	
			else {
				fprintf(ofile, "%s,\n",keyWords[i].funcName);	
				t = 0;
			}
		}
	}
	fprintf(ofile, "};\n#endif\n");
	fclose(ofile);

	// Output Switch Statement
//	ofile = fopen("U-KEYWRD.C","w");

//	fprintf(ofile, "// Switch Statement\n");
//	fprintf(ofile, "// Automatically generated by ParserMaker.c\n");
//	fprintf(ofile, "//\n");
//	fprintf(ofile, "    keyVal = -1L;\n");
//	GenSwitch(0,0);

//	fclose(ofile);

	// Output Parse Table
	// SortKeywords();

	ofile = fopen("U-PTABLE.C","w");
	// setbuf(ofile,NULL);
	fprintf(ofile, "// Parse Table\n");
	fprintf(ofile, "// Automatically generated by ParserMaker.c\n");
	fprintf(ofile, "//\n");
	fprintf(ofile, "unsigned char gParseTab[] = {\n");
	GenPTable(0,0);
	fprintf(ofile, "};");
	fclose(ofile);
}

short	CountMatches(char *input, short level);
short	CountMatches(char *input, short level)
{
	short	i,n=0;
	long	alphaMatch = 0L;
	
	for (i = 0; keyWords[i].keyWord; ++i) {
		if (level == 0 || strncmp(keyWords[i].keyWord,input,level) == 0) {
			if (!(alphaMatch & (1L << keyWords[i].keyWord[level]))) {
				++n;
				alphaMatch |= (1L << keyWords[i].keyWord[level]);
			}
		}
	}
	return n;
}

#define Margin	4

void GenSwitch(short input, short level)
{
	short	i,j;
	long	alphaBits = 0L;

	if (CountMatches(keyWords[input].keyWord,level) == 1) {
		if (keyWords[input].keyWord[level] == 0)
			fprintf(ofile, "%*sif (*(sp++) == \'\\0\')\n%*skeyVal = sf_%s;\n",
				level+Margin, "",
				level+1+Margin,"",&keyWords[input].funcName[2]);
		else {
			fprintf(ofile, "%*sif (*(sp++) == \'%c\')\n",level+Margin,"",keyWords[input].keyWord[level]);
			GenSwitch(input,level+1);
		}
	}
	else {
		fprintf(ofile, "%*sswitch (*(sp++)) {\n",level+Margin,"");
	
		for (i = 0; keyWords[i].keyWord; ++i) {
			if (level == 0 || strncmp(keyWords[i].keyWord,keyWords[input].keyWord,level) == 0) {
				// Match thus far
				if (keyWords[i].keyWord[level] == 0)
					fprintf(ofile, "%*scase \'\\0\': keyVal = sf_%s; break;\n",
						level+Margin, "",&keyWords[i].funcName[2]);
				else if (!(alphaBits & (1L << (keyWords[i].keyWord[level]-'A')))) {
					alphaBits |= (1L << (keyWords[i].keyWord[level]-'A'));
					fprintf(ofile, "%*scase \'%c\':\n", 
						level+Margin, "",keyWords[i].keyWord[level]);
						GenSwitch(i,level+1);
					fprintf(ofile, "%*sbreak;\n",(level+1)+Margin, ""); 
				}
			}
		}
		fprintf(ofile, "%*s}\n",level+Margin, "");
	}
}

long	gLineNbr = 0;

void LinkLastOffset(long offset, long lastLink);
void LinkLastOffset(long offset, long lastLink)
{
	fseek(ofile, offset, SEEK_SET);
	fprintf(ofile,"%02lx",gLineNbr - lastLink);
	fseek(ofile, 0, SEEK_END);
	if (gLineNbr - lastLink > 255) {
		printf("Offset: %ld\n",gLineNbr - lastLink);
		SysBeep(1);
	}
}

void GenPTable(short input, short level)
{
	short	i,j,matches,matchCnt=0;
	long	alphaBits = 0L;
	long	lastOffset = -1L,linkIndex;

	matches = CountMatches(keyWords[input].keyWord,level);
	if (matches >= 1) {
		// fprintf(ofile, "%*sswitch (*(sp++)) {\n",level+Margin,"");
		matchCnt = 0;
		for (i = 0; keyWords[i].keyWord; ++i) {
			// 1/31/97 JAB Added missing parenthesis to the following expression
			if ((level == 0 || strncmp(keyWords[i].keyWord,keyWords[input].keyWord,level) == 0) &&
				(keyWords[i].keyWord[level] == 0 || !(alphaBits & (1L << (keyWords[i].keyWord[level]-'A'))))) {
				// if lastOffset, link it
				if (lastOffset != -1) {
					LinkLastOffset(lastOffset,linkIndex);
				}
				// Match thus far
				if (matchCnt == matches-1) {
					if (keyWords[i].keyWord[level] == 0) {
						fprintf(ofile, "%*s\'\\0\',\n%*s    sf_%s,\n",
										level*2+Margin,"",level*2+Margin,"",&keyWords[i].funcName[2]);
						gLineNbr += 2;
						++matchCnt;
					}
					else if (!(alphaBits & (1L << (keyWords[i].keyWord[level]-'A')))) {
						alphaBits |= (1L << (keyWords[i].keyWord[level]-'A'));
						fprintf(ofile, "%*s\'%c\',\n",level*2+Margin,"",keyWords[i].keyWord[level]);
						gLineNbr++;
						GenPTable(i,level+1);
						++matchCnt;
					}
				}
				else {
					if (keyWords[i].keyWord[level] == 0) {
						linkIndex = gLineNbr;
						fprintf(ofile, "%*sP_MatchFork | \'\\0\', 0x??,\n",
							level*2+Margin, "");
						lastOffset = ftell(ofile) - 4;
					
						fprintf(ofile,"%*s    sf_%s,\n",level*2+Margin, "",
							&keyWords[i].funcName[2]);
						gLineNbr += 3;
						++matchCnt;
					}
					else if (!(alphaBits & (1L << (keyWords[i].keyWord[level]-'A')))) {
						linkIndex = gLineNbr;
						alphaBits |= (1L << (keyWords[i].keyWord[level]-'A'));
						fprintf(ofile, "%*sP_MatchFork | \'%c\', 0x??,\n", 
							level*2+Margin, "",keyWords[i].keyWord[level]);
						lastOffset = ftell(ofile)- 4;
						gLineNbr += 2;
						GenPTable(i,level+1);
						// Generate Offset...
						++matchCnt;
					}
				}
			}
		}
	}
	if (input == 0 && level == 0)
		fprintf(ofile, "// %ld Lines\n",gLineNbr);
}


void SortKeywords()
{
	KeyWords	temp;
	short		i,j;
	for (i = 1; keyWords[i].keyWord; ++i) {
		temp = keyWords[i];
		for (j = i; j > 0 && strcmp(temp.keyWord,keyWords[j-1].keyWord) < 0; --j) {
			keyWords[j] = keyWords[j-1];
		}
		if (i != j)
			keyWords[j] = temp;
	}
}
